@@ -1,23 +1,24 @@
-//! Lossless parser for Debian control files.
+#![deny(missing_docs)]
+//! Parser for Debian control files.
 //!
-//! This crate provides a parser for Debian control files. It is lossless, meaning that it will
-//! preserve the original formatting of the file. It also provides a way to serialize the parsed
-//! data back to a string.
+//! This crate provides a parser for Debian control files.
 //!
 //! # Example
 //!
 //! ```rust
-//! use debian_control::{Control, Priority};
+//! use debian_control::lossy::Control;
+//! use debian_control::fields::Priority;
 //! use std::fs::File;
 //!
 //! let mut control = Control::new();
-//! let mut source = control.add_source("hello");
-//! source.set_section(Some("rust"));
+//! let mut source = &mut control.source;
+//! source.name = "hello".to_string();
+//! source.section = Some("rust".to_string());
 //!
 //! let mut binary = control.add_binary("hello");
-//! binary.set_architecture(Some("amd64"));
-//! binary.set_priority(Some(Priority::Optional));
-//! binary.set_description(Some("Hello, world!"));
+//! binary.architecture = Some("amd64".to_string());
+//! binary.priority = Some(Priority::Optional);
+//! binary.description = Some("Hello, world!".to_string());
 //!
 //! assert_eq!(control.to_string(), r#"Source: hello
 //! Section: rust
@@ -28,18 +29,27 @@
 //! Description: Hello, world!
 //! "#);
 //! ```
-pub mod apt;
-pub mod changes;
-pub mod control;
-pub use control::{Binary, Control, Source};
+//!
+//! See the ``lossless`` module for a parser that preserves all comments and formatting, and
+//! as well as allowing inline errors.
+pub mod lossy;
+pub use lossless::control::{Binary, Control, Source};
 pub mod fields;
 pub use fields::*;
+pub mod lossless;
+pub use lossless::apt;
+pub use lossless::changes;
+pub use lossless::control;
 pub mod pgp;
 pub mod relations;
 pub mod vcs;
 
+use std::borrow::Cow;
+
+/// Error type for parsing an identity string.
 #[derive(Debug, PartialEq)]
 pub enum ParseIdentityError {
+    /// No email address found.
     NoEmail,
 }
 
@@ -84,6 +94,40 @@ pub fn parse_identity(s: &str) -> Result<(&str, &str), ParseIdentityError> {
     }
 }
 
+/// A trait for looking up versions of packages.
+pub trait VersionLookup {
+    /// Look up the version of a package.
+    fn lookup_version<'a>(
+        &'a self,
+        package: &'_ str,
+    ) -> Option<std::borrow::Cow<'a, debversion::Version>>;
+}
+
+impl VersionLookup for std::collections::HashMap<String, debversion::Version> {
+    fn lookup_version<'a>(&'a self, package: &str) -> Option<Cow<'a, debversion::Version>> {
+        self.get(package).map(Cow::Borrowed)
+    }
+}
+
+impl<F> VersionLookup for F
+where
+    F: Fn(&str) -> Option<debversion::Version>,
+{
+    fn lookup_version<'a>(&'a self, name: &str) -> Option<Cow<'a, debversion::Version>> {
+        self(name).map(Cow::Owned)
+    }
+}
+
+impl VersionLookup for (String, debversion::Version) {
+    fn lookup_version<'a>(&'a self, name: &str) -> Option<Cow<'a, debversion::Version>> {
+        if name == self.0 {
+            Some(Cow::Borrowed(&self.1))
+        } else {
+            None
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
